# from src.model import utterance
# from src.services.answer import compute_answer
import time
import requests
from huma_sdk.utils.log_utils import get_logger
from huma_sdk.config import Config
import json
from typing import Callable, List
from huma_sdk.utils.token_manager import TokenManager
from huma_sdk.config import Config

class huma_sdk():
    def __init__(self, customer=Config.HUMA_CUSTOMER, environment=Config.HUMA_ENVIRONMENT_KEY, url=Config.HUMA_API_URL, *args, **kwargs):
        self.customer = self._get_default(customer, Config.HUMA_CUSTOMER)
        self.environment = self._get_default(environment, Config.HUMA_ENVIRONMENT_KEY)
        self._run_validations()
        if kwargs.get("cli_token"):
            self.token_manager = TokenManager(customer=self.customer, environment=self.environment, cli_token=kwargs.get("cli_token"))
        else:
            self.token_manager = TokenManager(customer=self.customer, environment=self.environment)
        self.logger = get_logger(__name__ + " " + self.customer + "-" + self.environment)
        self.url = self.token_manager.get_url()

    def _get_default(self, var, default):
        if not var:
            return default
        return var

    def _run_validations(self):
        self._validate_customer()
        self._validate_environment()

    def _validate_customer(self):
        if not self.customer:
            raise ValueError(
                'You must either set environment variable HUMA_ENVIRONMENT_KEY or instantiate this library with environment={environment}')

    def _validate_environment(self):
        if not self.environment:
            raise ValueError(
                'You must either set environment variable HUMA_CUSTOMER or instantiate this library with customer={customer}')

    def _get_utterance(self, utterance: str):
        ''' function to get an utterance from the huma plataform'''
        token, error = self.token_manager.get_token()
        headers = {
            "Authorization": f"Bearer {token}",
            "Content-Type": "application/json"
        }
        payload = {
            "utterance": utterance,
            "apps": "az-gma-pilot-query"
        }

        up_response = requests.post(
            self.url + "answer/get_utterance_response", headers=headers, json=payload)
        self.logger.debug(f"on utterance '{utterance}' huma-server returned up response: {up_response}")
        job_request = json.loads(up_response.text)
        return job_request

    def send_smoke_notification(self,error_list,test_name,customer,environment):
        token, error = self.token_manager.get_token()
        headers = {
            "Authorization": f"Bearer {token}",
            "Content-Type": "application/json"
        }
        payload = {
            "error_list":error_list,
            "test_name":test_name,
            "customer":customer,
            "environment":environment
        }
        up_response = requests.post(
            self.url + "/send-smoke-notification", headers=headers, json=payload)

    def get_utterance(self, utterance: str):
        start_time = time.perf_counter()
        payload = None
        try:
            job_request = self._get_utterance(utterance=utterance)
            payload = job_request.get("up_response")
        except requests.ConnectionError:
            error = "huma-server connection error"
            self.logger.error(error)
        except requests.Timeout:
            error = f"huma-server timed out when trying to process utterance {utterance}"
            self.logger.error(error)
        except Exception as e:
            error = f"huma-server failed with {(e.__class__)} when processing utterance {utterance}"
            self.logger.error(error)

        total_time = time.perf_counter() - start_time
        self.logger.debug(f"on utterance '{utterance}' get utterance payload completed in {total_time} time")
        return payload

    def logout(self) -> Callable:
        logged_out = self.token_manager.logout()
        return logged_out

    def login(self) -> Callable:
        logged_in, error = self.token_manager.login()
        return logged_in

    def _get_answer(self, job_id: str):
        token, error = self.token_manager.get_token()
        if error:
            return {"msg": "there is no valid access token.  Please check your credentials"}
        headers = {
            "Authorization": f"Bearer {token}",
            "Content-Type": "application/json"
        }
        # pass command so server knows this is cli activity
        payload = {"job_id": job_id}
        answer_request = requests.get(self.url + "answer/get_answer", headers=headers, json=json.dumps(payload))
        return answer_request

    def get_answer(self, job_id: str):
        start_time = time.perf_counter()
        error = None
        payload = None
        try:
            # calls _submit_question() function
            # pass command so server knows this is cli activity
            answer_request = self._get_answer(job_id=job_id)
            payload = json.loads(answer_request.text)
        except requests.ConnectionError:
            error = "huma-server connection error"
            self.logger.error(error)
        except requests.Timeout:
            error_text = f"huma-server timed out when trying to get answer to job {job_id}"
            self.logger.error(error)
        except Exception as e:
            error = f"huma-server failed with {(e.__class__)} when processing job {job_id}"
            self.logger.error(error)

        total_time = time.perf_counter() - start_time
        self.logger.debug(f"submit question completed in {total_time} time")

        return payload

    def get_suggestions_query(self, partial_query: str):
        ''' function that will call get_suggestions() from answer.py within huma-server '''
        token, error = self.token_manager.get_token()
        headers = {
            "Authorization": f"Bearer {token}",
            "Content-Type": "application/json"
        }
        params = {
            "partialQuestion": partial_query
        }

        suggestions_payload = None

        try:
            # has suggestions key already
            response = requests.get(self.url + "suggestions", headers=headers, params=params)
            suggestions_payload = json.loads(response.text)
        except requests.ConnectionError:
            error = "huma-server connection error"
            self.logger.error(error)
        except requests.Timeout:
            error = f"huma-server timed out when trying to get a list of all suggestions."
            self.logger.error(error)
        except Exception as e:
            error = f"huma-server failed with {(e.__class__)} when trying to get a list of all suggestions."
            self.logger.error(error)

        return suggestions_payload

    def get_suggestions(self):
        ''' function that will call get_suggestions() from answer.py within huma-server '''
        token, error = self.token_manager.get_token()
        headers = {
            "Authorization": f"Bearer {token}",
            "Content-Type": "application/json"
        }
        params = {
            "allSuggestions": True
        }

        suggestions_payload = None

        try:
            # has suggestions key already
            response = requests.get(self.url + "suggestions", headers=headers, params=params)
            suggestions_payload = json.loads(response.text)
        except requests.ConnectionError:
            error = "huma-server connection error"
            self.logger.error(error)
        except requests.Timeout:
            error = f"huma-server timed out when trying to get a list of all suggestions."
            self.logger.error(error)
        except Exception as e:
            error = f"huma-server failed with {(e.__class__)} when trying to get a list of all suggestions."
            self.logger.error(error)

        return suggestions_payload

    def create_suggestions(self, suggestions: list):
        ''' function that will call get_suggestions() from answer.py within huma-server '''
        token, error = self.token_manager.get_token()
        headers = {
            "Authorization": f"Bearer {token}",
            "Content-Type": "application/json"
        }

        payload = {
            "suggestions": suggestions
        }

        response = None

        try:
            response = requests.post(self.url + "suggestions", headers=headers, json=payload)
        except requests.ConnectionError:
            error = "huma-server connection error"
            self.logger.error(error)
        except requests.Timeout:
            error = f"huma-server timed out when trying to update suggestions."
            self.logger.error(error)
        except Exception as e:
            error = f"huma-server failed with {(e.__class__)} when creating suggestion list"
            self.logger.error(error)

        return response

    def _submit_question(self, question: str):
        ''' function to submit a question to huma plataform'''
        # call API and get back job_id
        token, error = self.token_manager.get_token()
        if error:
            return {"msg": "there is no valid access token.  Please check your credentials"}
        headers = {
            "Authorization": f"Bearer {token}",
            "Content-Type": "application/json"
        }
        payload = {
            "utterance": question
        }

        job_request = requests.post(self.url + "answer", headers=headers, json=payload)

        self.logger.debug("huma-server returned: {}".format(job_request))
        return job_request

    def _complete_async_answer(self, utteranceIds: List[str]):
        ''' function to submit a question to huma plataform'''
        # call API and get back job_id
        token, error = self.token_manager.get_token()
        if error:
            return {"msg": "there is no valid access token.  Please check your credentials"}
        headers = {
            "Authorization": f"Bearer {token}",
            "Content-Type": "application/json"
        }
        payload = {
            "utteranceIds": utteranceIds
        }

        job_request = requests.post(self.url + "answer/complete-async", headers=headers, json=payload)

        self.logger.debug("huma-server returned: {}".format(job_request))
        return job_request

    def complete_async_answers(self, utteranceIds: List[str]):
        start_time = time.perf_counter()
        payload = {}
        try:
            # calls _submit_question() function
            # pass command so server knows this is cli activity
            job_request = self._complete_async_answer(utteranceIds=utteranceIds)
            if job_request.status_code == 504:
                error = "huma-server gateway timeout"
                self.logger.error(error)
            elif job_request.status_code == 500:
                error = payload.get("message")
                self.logger.error(error)
            else:
                payload = json.loads(job_request.text)
        except requests.ConnectionError:
            error = "huma-server connection error"
            self.logger.error(error)
        except requests.Timeout:
            error = f"huma-server timed out when trying to process question."
            self.logger.error(error)
        except Exception as e:
            error = f"huma-server failed with {(e.__class__)} when processing question."
            self.logger.error(error)

        total_time = time.perf_counter() - start_time
        self.logger.debug(f"submit question completed in {total_time} time")

        return payload

    def submit_question(self, question: str):
        start_time = time.perf_counter()
        payload = None
        try:
            # calls _submit_question() function
            # pass command so server knows this is cli activity
            job_request = self._submit_question(question=question)
            payload = json.loads(job_request.text)

            if job_request.status_code == 500:
                payload["utterance"]=question
                error = payload.get("message")
                self.logger.error(error)
        except requests.ConnectionError:
            error = "huma-server connection error"
            self.logger.error(error)
        except requests.Timeout:
            error = f"huma-server timed out when trying to process question {question}"
            self.logger.error(error)
        except Exception as e:
            error = f"huma-server failed with {(e.__class__)} when processing question {question}"
            self.logger.error(error)

        total_time = time.perf_counter() - start_time
        self.logger.debug(f"submit question completed in {total_time} time")

        return payload

    def get_audit_trail(self):
        token, error = self.token_manager.get_token()
        headers = {
            "Authorization": f"Bearer {token}",
            "Content-Type": "application/json"
        }

        try:
            audit_trail = requests.get(self.url + "audit", headers=headers)
            # convert to json format
            audit_trail = json.loads(audit_trail.text)
        except requests.ConnectionError:
            error = "huma-server connection error"
            self.logger.error(error)
        except requests.Timeout:
            error = f"huma-server timed out when trying to get audit trail"
            self.logger.error(error)
        except Exception as e:
            error = f"huma-server failed with {(e.__class__)} when processing audit trail"
            self.logger.error(error)
        # returning in JSON format
        return audit_trail

    def get_activity_dump(self, duration:int):
        token, error = self.token_manager.get_token()
        headers = {
            "Authorization": f"Bearer {token}",
            "Content-Type": "application/json"
        }
        activity = "{}"
        # create valid duration format from int. Duration of how many days to retrieve data from i.e the last n days of activity
        duration = (str(duration)+"d")
        payload = {"duration":duration}

        try:
            request = requests.get(self.url + "dump", headers=headers, json=payload)
            # convert to json format
            activity = json.loads(request.text)
        except requests.ConnectionError:
            error = "huma-server connection error"
            self.logger.error(error)
        except requests.Timeout:
            error = f"huma-server timed out when trying to get activity log"
            self.logger.error(error)
        except Exception as e:
            error = f"huma-server failed with {(e.__class__)} when processing activity log"
            self.logger.error(error)
        # returning in JSON format
        return activity

    def get_user_activity(self, duration:int):
        token, error = self.token_manager.get_token()
        headers = {
            "Authorization": f"Bearer {token}",
            "Content-Type": "application/json"
        }
        activity = "{}"
        duration = (str(duration)+"d")
        payload = {"duration":duration}
        try:
            request = requests.post(self.url + "activity", headers=headers, json=payload)
            # convert to json format
            activity = json.loads(request.text)
        except requests.ConnectionError:
            error = "huma-server connection error"
            self.logger.error(error)
        except requests.Timeout:
            error = f"huma-server timed out when trying to get activity log"
            self.logger.error(error)
        except Exception as e:
            error = f"huma-server failed with {(e.__class__)} when processing activity log"
            self.logger.error(error)
        # returning in JSON format
        return activity

    # get_redis_keys
    def get_redis_keys(self, namespace_filter:str="*") -> dict:
        '''get a list of all keys that meet the namespace filter'''
        token, error = self.token_manager.get_token()
        headers = {
            "Authorization": f"Bearer {token}"
        }
        #TODO: need to add pagination
        #page_num =  1
        #num_per_page = 100

        try:
            response = requests.get(self.url + "redis-keys" + f"?namespace_filter={namespace_filter}", headers=headers)
            # convert to json format
            if response.status_code == 200:
                payload = json.loads(response.text)
                return payload
            if response.status_code == 204:
                payload = {"redis_keys": "", "msg": "redis keys do not exist on server.", }
                return payload
        except requests.ConnectionError:
            error = "huma-server connection error"
            redis_keys = {"msg": error}
            self.logger.error(error)
        except requests.Timeout:
            error = f"huma-server timed out when trying to get redis keys"
            redis_keys = {"msg": error}
            self.logger.error(error)
        except Exception as e:
            error = f"huma-server failed with {(e.__class__)} when processing get redis keys"
            redis_keys = {"msg": error}
            self.logger.error(error)
        # returning in JSON format
        return redis_keys

    # get_redis_key
    def get_redis_key(self, key:str) -> dict:
        '''get redis key'''
        token, error = self.token_manager.get_token()
        headers = {
            "Authorization": f"Bearer {token}"
        }
        if type(key) != str or key is None:
            return {}

        try:
            response = requests.get(self.url + "redis" + f"?key={key}", headers=headers)
            # convert to json format
            if response.status_code == 200:
                payload = json.loads(response.text)
                return payload
            if response.status_code == 204:
                payload = {"redis_key": "", "msg": "redis key does not exist on server.", }
                return payload
        except requests.ConnectionError:
            error = "huma-server connection error"
            redis_key = {"msg": error}
            self.logger.error(error)
        except requests.Timeout:
            error = f"huma-server timed out when trying to get redis key"
            redis_key = {"msg": error}
            self.logger.error(error)
        except Exception as e:
            error = f"huma-server failed with {(e.__class__)} when processing get redis key"
            redis_key = {"msg": error}
            self.logger.error(error)
        # returning in JSON format
        return redis_key

    def put_redis_key(self, key: str, value: dict) -> bool:
        '''function that will put a value in a redis key '''
        msg = "You must specify a value for key and value in a json payload."
        token, error = self.token_manager.get_token()
        headers = {
            "Authorization": f"Bearer {token}"
        }

        response = None
        redis_res = None

        try:
            response = requests.post(self.url + "redis", headers=headers, json={"key": key, "value": value})

            # convert to json format
            if response.status_code == 200:
                payload = json.loads(response.text)
                return payload
            if response.status_code == 400:
                payload = {"msg": "You must specify a value for key and value in a json payload.", }
                return payload

        except requests.ConnectionError:
            error = "huma-server connection error"
            redis_res = {"msg": error}
            self.logger.error(error)
        except requests.Timeout:
            error = f"huma-server timed out when trying to add redis key"
            redis_res = {"msg": error}
            self.logger.error(error)
        except Exception as e:
            error = f"huma-server failed with {(e.__class__)} when putting redis key and value"
            redis_res = {"msg": error}
            self.logger.error(error)

        return redis_res

    # clear_redis_key's
    def clear_redis_keys(self, namespace:str) -> dict:
        '''clear redis key'''
        token, error = self.token_manager.get_token()
        headers = {
            "Authorization": f"Bearer {token}"
        }

        if type(namespace) != str or namespace is None:
            return {}

        try:
            response = requests.get(self.url + "clear-redis-keys" + f"?namespace={namespace}", headers=headers)

            # convert to json format
            if response.status_code == 200:
                return json.loads(response.text)
            if response.status_code == 204:
                return {"msg": "Redis Keys does not exist on server with namespace "+ namespace, }
        except requests.ConnectionError:
            error = "huma-server connection error"
            redis_key = {"msg": error}
            self.logger.error(error)
        except requests.Timeout:
            error = f"huma-server timed out when trying to clean redis key"
            redis_key = {"msg": error}
            self.logger.error(error)
        except Exception as e:
            error = f"huma-server failed with {(e.__class__)} when processing clean redis key"
            redis_key = {"msg": error}
            self.logger.error(error)
        # returning in JSON format
        return redis_key
        
    def get_up_config(self, version:int) -> dict:
        '''save up config to s3 where up will use it.'''
        token, error = self.token_manager.get_token()
        headers = {
            "Authorization": f"Bearer {token}"
        }
        up_config = {}
        up_files = None
        try:
            response = requests.get(self.url + "up_config" + "?is_yaml=true" + "&nth_version=" + str(version), headers=headers)
            # convert to json format
            if response.status_code == 200:
                payload = json.loads(response.text)
                #dict_payload = json.loads(payload)
                return payload
            if response.status_code == 204:
                payload = {"up_config": "", "msg": "config file does not exist on server.", }
                return payload
        except requests.ConnectionError:
            error = "huma-server connection error"
            up_files = {"msg": error}
            self.logger.error(error)
        except requests.Timeout:
            error = f"huma-server timed out when trying to get up config"
            up_files = {"msg": error}
            self.logger.error(error)
        except Exception as e:
            error = f"huma-server failed with {(e.__class__)} when processing get up config"
            up_files = {"msg": error}
            self.logger.error(error)
        # returning in JSON format
        return up_files

    def put_up_config(self, up_config: dict) -> bool:
        ''' function that will call put_up_config_to_s3() from up.py within huma-server '''
        token, error = self.token_manager.get_token()
        headers = {
            "Authorization": f"Bearer {token}"
        }

        response = None

        try:
            response = requests.post(self.url + "up_config", headers=headers, json=up_config)
        except requests.ConnectionError:
            error = "huma-server connection error"
            self.logger.error(error)
        except requests.Timeout:
            error = f"huma-server timed out when trying to update suggestions."
            self.logger.error(error)
        except Exception as e:
            error = f"huma-server failed with {(e.__class__)} when putting up config"
            self.logger.error(error)

        return response

    def get_analyzer_logs(self, job_id: str):
        token, error = self.token_manager.get_token()
        headers = {
            "Authorization": f"Bearer {token}",
            "Content-Type": "application/json"
        }
        params = {
            "job_id": job_id
        }
        logs = {}
        try:
            request = requests.get(self.url + "analyzer_log", headers=headers, params=params)
            # convert to json format
            if request.status_code == 200:
                logs = json.loads(request.text)
        except requests.ConnectionError:
            error = "huma-server connection error"
            self.logger.error(error)
        except requests.Timeout:
            error = f"huma-server timed out when trying to get analyzer log"
            self.logger.error(error)
        except Exception as e:
            error = f"huma-server failed with {(e.__class__)} when processing get analyzer log"
            self.logger.error(error)
        # returning in JSON format
        return logs

    def _get_quick_links(self):
        token, errror = self.token_manager.get_token()
        headers = {
            "Authorization": f"Bearer {token}"
        }
        quick_links = {}
        try:
            request = requests.get(self.url + "quick-links/true", headers=headers)
            # convert to json format
            quick_links = json.loads(request.text)
        except requests.ConnectionError:
            error = "huma-server connection error"
            self.logger.error(error)
        except requests.Timeout:
            error = f"huma-server timed out when trying to get activity log"
            self.logger.error(error)
        except Exception as e:
            error = f"huma-server failed with {(e.__class__)} when processing activity log"
            self.logger.error(error)
        # returning in JSON format
        return quick_links

    def get_quick_links(self):
        quick_links = self._get_quick_links()
        return quick_links

    # Added by shani
    def get_valid_domains(self):
        token, error = self.token_manager.get_token()
        headers = {
            "Authorization": f"Bearer {token}",
            "Content-Type": "application/json"
        }

        valid_domains = []
        try:
            request = requests.get(self.url + "get-site-settings", headers=headers)
            # convert to json format
            response = json.loads(request.text)
            valid_domains = response.get('valid_domains')
        except requests.ConnectionError:
            error = "huma-server connection error"
            self.logger.error(error)
        except requests.Timeout:
            error = f"huma-server timed out when trying to get data."
            self.logger.error(error)
        except Exception as e:
            error = f"huma-server failed with {(e.__class__)} when processing valid domains log"
            self.logger.error(error)
        # returning in JSON format
        return valid_domains

    def get_ecs_services_status(self):
        token, error = self.token_manager.get_token()
        headers = {
            "Authorization": f"Bearer {token}",
            "Content-Type": "application/json"
        }

        results = []
        try:
            request = requests.get(self.url + "ecs-services-status", headers=headers)
            # convert to json format
            response = json.loads(request.text)

            if request.status_code == 401:
                error = "auth error"
                self.logger.error(error)
            if request.status_code == 500:
                error = response.get("message", "huma server error")
                self.logger.error(error)

            results = response.get('results',results)
        except requests.ConnectionError:
            error = "huma-server connection error"
            self.logger.error(error)
        except requests.Timeout:
            error = f"huma-server timed out when trying to get data."
            self.logger.error(error)
        except Exception as e:
            error = f"huma-server failed with {(e.__class__)} when processing ecs services status"
            self.logger.error(error)
        # returning in JSON format
        return results

    def get_batch_cpu_status(self):
        token, error = self.token_manager.get_token()
        headers = {
            "Authorization": f"Bearer {token}",
            "Content-Type": "application/json"
        }

        results = []
        try:
            request = requests.get(self.url + "batch-cpu-status", headers=headers)

            # convert to json format
            response = json.loads(request.text)

            if request.status_code == 401:
                error = "auth error"
                self.logger.error(error)
            if request.status_code == 500:
                error = response.get("message", "huma server error")
                self.logger.error(error)

            results = response.get('results',results)
        except requests.ConnectionError:
            error = "huma-server connection error"
            self.logger.error(error)
        except requests.Timeout:
            error = f"huma-server timed out when trying to get data."
            self.logger.error(error)
        except Exception as e:
            error = f"huma-server failed with {(e.__class__)} when processing ecs services status"
            self.logger.error(error)
        # returning in JSON format
        return results

    def get_batch_recent_job_definition(self):
        token, error = self.token_manager.get_token()
        headers = {
            "Authorization": f"Bearer {token}",
            "Content-Type": "application/json"
        }
        results = []
        try:
            
            request = requests.get(self.url + "batch-recent-job-definition", headers=headers)
            if request.status_code == 401:
                error = "auth error"
                self.logger.error(error)
            # convert to json format
            response = json.loads(request.text)
            results = response.get('results', results)
        except requests.ConnectionError:
            error = "huma-server connection error"
            self.logger.error(error)
        except requests.Timeout:
            error = f"huma-server timed out when trying to get data."
            self.logger.error(error)    
        except Exception as e:
            error = f"huma-server failed with {(e.__class__)} when processing ecs services status"
            self.logger.error(error)
        # returning in JSON format
        return results

    def get_mongodb_connection_status(self):
        token, error = self.token_manager.get_token()
        headers = {
            "Authorization": f"Bearer {token}",
            "Content-Type": "application/json"
        }
        result = {}

        try:
            request = requests.get(self.url + "mongo-db-connection", headers=headers)
            if request.status_code == 401:
                error = "auth error"
                self.logger.error(error)
            # convert to json format
            result = json.loads(request.text)
        except requests.ConnectionError:
            error = "huma-server connection error"
            self.logger.error(error)
        except requests.Timeout:
            error = f"huma-server timed out when trying to get data."
            self.logger.error(error)
        except Exception as e:
            error = f"huma-server failed with {(e.__class__)} when connection mongo db"
            self.logger.error(error)
        # returning in JSON format
        return result

    def get_ctgov_db_status(self):
        token, error = self.token_manager.get_token()
        headers = {
            "Authorization": f"Bearer {token}",
            "Content-Type": "application/json"
        }
        result = {}

        try:
            request = requests.get(self.url + "ct-gov-database-status", headers=headers)
            if request.status_code == 401:
                error = "auth error"
                self.logger.error(error)
            result = json.loads(request.text)
        except requests.ConnectionError:
            error = "huma-server connection error"
            self.logger.error(error)
        except requests.Timeout:
            error = f"huma-server timed out when trying to get data."
            self.logger.error(error)
        except Exception as e:
            error = f"huma-server failed with {(e.__class__)} when connection ct-gov db"
            self.logger.error(error)
        # returning in JSON format
        return result

    def get_client_db_status(self, db_name):
        token, error = self.token_manager.get_token()
        headers = {
            "Authorization": f"Bearer {token}",
            "Content-Type": "application/json"
        }
        result = {}

        try:
            url = f"{self.url}client-database-status/{db_name}"
            request = requests.get(url, headers=headers)
            if request.status_code == 401:
                error = "auth error"
                self.logger.error(error)
                return {'status':False, 'error': error }

            result = json.loads(request.text)
        except requests.ConnectionError:
            error = "huma-server connection error"
            self.logger.error(error)
        except requests.Timeout:
            error = f"huma-server timed out when trying to get data."
            self.logger.error(error)
        except Exception as e:
            error = f"huma-server failed with {(e.__class__)} when connection client db"
            self.logger.error(error)
        # returning in JSON format
        return result

    def get_minimum_vcpu(self):
        token, error = self.token_manager.get_token()
        headers = {
            "Authorization": f"Bearer {token}",
            "Content-Type": "application/json"
        }
        result = {}

        try:
            request = requests.get(self.url + "get-compute-minimum-vcpus", headers=headers)
            if request.status_code == 401:
                error = "auth error"
                self.logger.error(error)

            # convert to json format
            result = json.loads(request.text)
        except requests.ConnectionError:
            error = "huma-server connection error"
            self.logger.error(error)
        except requests.Timeout:
            error = f"huma-server timed out when trying to get data."
            self.logger.error(error)
        except Exception as e:
            error = f"huma-server failed with {(e.__class__)} when retrieving minimum vCPUs from compute environment"
            self.logger.error(error)
        # returning in JSON format
        return result

    def set_minimum_vcpu(self, minvcpu: int):
        token, error = self.token_manager.get_token()
        headers = {
            "Authorization": f"Bearer {token}",
            "Content-Type": "application/json"
        }
        result = {}

        try:
            params = {
                "minimum_vcpus": minvcpu
            }
            request = requests.post(self.url + "set-compute-minimum-vcpus", headers=headers, json=params)
            if request.status_code == 401:
                error = "auth error"
                self.logger.error(error)

            # convert to json format
            result = json.loads(request.text)
        except requests.ConnectionError:
            error = "huma-server connection error"
            self.logger.error(error)
        except requests.Timeout:
            error = f"huma-server timed out when trying to get data."
            self.logger.error(error)
        except Exception as e:
            error = f"huma-server failed with {(e.__class__)} when retrieving minimum vCPUs from compute environment"
            self.logger.error(error)
        # returning in JSON format
        return result

    def update_quicklinks_reporting(self, data:object):
        token, error = self.token_manager.get_token()
        headers = {
            "Authorization": f"Bearer {token}",
            "Content-Type": "application/json"
        }
        result = {}

        try:
            params = data
            request = requests.post(self.url + "report-quick-links", headers=headers, json=params)
            if request.status_code == 401:
                error = "auth error"
                self.logger.error(error)

            # convert to json format
            result = json.loads(request.text)
        except requests.ConnectionError:
            error = "huma-server connection error"
            self.logger.error(error)
        except requests.Timeout:
            error = f"huma-server timed out when trying to get data."
            self.logger.error(error)
        except Exception as e:
            error = f"huma-server failed with {(e.__class__)} when updating quicklinks errors"
            self.logger.error(error)
        # returning in JSON format
        return result

    def get_redis_connection_test(self):
        token, error = self.token_manager.get_token()
        headers = {
            "Authorization": f"Bearer {token}",
            "Content-Type": "application/json"
        }
        result = {}

        try:
            request = requests.get(self.url + "check-redis-status", headers=headers)
            if request.status_code == 401:
                error = "auth error"
                self.logger.error(error)
            result = json.loads(request.text)
        except requests.ConnectionError:
            error = "huma-server connection error"
            self.logger.error(error)
        except requests.Timeout:
            error = f"huma-server timed out when trying to get data."
            self.logger.error(error)
        except Exception as e:
            error = f"huma-server failed with {(e.__class__)} when fetching redis status"
            self.logger.error(error)
        # returning in JSON format
        return result

    def get_system_status(self):
        token, error = self.token_manager.get_token()
        headers = {
            "Authorization": f"Bearer {token}",
            "Content-Type": "application/json"
        }
        result = {}
        
        try:
            request = requests.get(self.url + "get-system-status", headers=headers)
            if request.status_code == 401:
                error = "auth error"
                self.logger.error(error)

            # convert to json format
            result = json.loads(request.text)
        except requests.ConnectionError:
            error = "huma-server connection error"
            self.logger.error(error)
        except requests.Timeout:
            error = f"huma-server timed out when trying to get data."
            self.logger.error(error)
        except Exception as e:
            error = f"huma-server failed with {(e.__class__)} when retrieving status of the system."
            self.logger.error(error)
        # returning in JSON format
        return result

    def get_system_activity(self):
        token, error = self.token_manager.get_token()
        headers = {
            "Authorization": f"Bearer {token}",
            "Content-Type": "application/json"
        }
        result = {}
        
        try:
            request = requests.get(self.url + "stop-system-if-inactive", headers=headers)
            if request.status_code == 401:
                error = "auth error"
                self.logger.error(error)

            # convert to json format
            result = json.loads(request.text)
        except requests.ConnectionError:
            error = "huma-server connection error"
            self.logger.error(error)
        except requests.Timeout:
            error = f"huma-server timed out when trying to get data."
            self.logger.error(error)
        except Exception as e:
            error = f"huma-server failed with {(e.__class__)} when trying to stop system on inactivty."
            self.logger.error(error)
        # returning in JSON format
        return result

    def stop_system_activity(self):
        token, error = self.token_manager.get_token()
        headers = {
            "Authorization": f"Bearer {token}",
            "Content-Type": "application/json"
        }
        result = {}
        
        try:
            request = requests.get(self.url + "stop-system", headers=headers)
            if request.status_code == 401:
                error = "auth error"
                self.logger.error(error)

            # convert to json format
            result = json.loads(request.text)
        except requests.ConnectionError:
            error = "huma-server connection error"
            self.logger.error(error)
        except requests.Timeout:
            error = f"huma-server timed out when trying to get data."
            self.logger.error(error)
        except Exception as e:
            error = f"huma-server failed with {(e.__class__)} when trying to stop system on inactivty."
            self.logger.error(error)
        # returning in JSON format
        return result

    def start_system_activity(self):
        token, error = self.token_manager.get_token()
        headers = {
            "Authorization": f"Bearer {token}",
            "Content-Type": "application/json"
        }
        result = {}
        
        try:
            request = requests.get(self.url + "start-system", headers=headers)
            if request.status_code == 401:
                error = "auth error"
                self.logger.error(error)

            # convert to json format
            result = json.loads(request.text)
        except requests.ConnectionError:
            error = "huma-server connection error"
            self.logger.error(error)
        except requests.Timeout:
            error = f"huma-server timed out when trying to get data."
            self.logger.error(error)
        except Exception as e:
            error = f"huma-server failed with {(e.__class__)} when trying to stop system on inactivty."
            self.logger.error(error)
        # returning in JSON format
        return result

    def get_latest_user_context(self, user_id: str):
        token, error = self.token_manager.get_token()
        headers = {
            "Authorization": f"Bearer {token}",
            "Content-Type": "application/json"
        }
        user_context = {}

        try:
            request = requests.get(self.url + "get-latest-user-context/" + user_id, headers=headers)
            if request.status_code == 200:
                user_context = json.loads(request.text)
            if request.status_code == 204:
                user_context = {"user_context": {}, "msg": "There is no user context for this user id.", }

        except requests.ConnectionError:
            error = "huma-server connection error"
            self.logger.error(error)
        except requests.Timeout:
            error = f"huma-server timed out when trying to get data."
            self.logger.error(error)
        except Exception as e:
            error = f"huma-server failed with {(e.__class__)} when retrieving latest user context for this user id."
            self.logger.error(error)
        return user_context

    def get_all_user_context(self, user_id: str):
        token, error = self.token_manager.get_token()
        headers = {
            "Authorization": f"Bearer {token}",
            "Content-Type": "application/json"
        }
        user_context = {}

        try:
            request = requests.get(self.url + "get-all-user-context/" + user_id, headers=headers)
            if request.status_code == 200:
                user_context = json.loads(request.text)
            if request.status_code == 204:
                user_context = {"user_context": {}, "msg": "There is no user context for this user id.", }

        except requests.ConnectionError:
            error = "huma-server connection error"
            self.logger.error(error)
        except requests.Timeout:
            error = f"huma-server timed out when trying to get data."
            self.logger.error(error)
        except Exception as e:
            error = f"huma-server failed with {(e.__class__)} when retrieving all user context for this user id."
            self.logger.error(error)
        return user_context
    
    def create_user_context(self, user_id: str, user_context: dict):
        token, error = self.token_manager.get_token()
        headers = {
            "Authorization": f"Bearer {token}",
            "Content-Type": "application/json"
        }
        params = user_context

        try:
            request = requests.post(self.url + "create-user-context/" + user_id, headers=headers, json=params)
        except requests.ConnectionError:
            error = "huma-server connection error"
            self.logger.error(error)
        except requests.Timeout:
            error = f"huma-server timed out when trying to post data."
            self.logger.error(error)
        except Exception as e:
            error = f"huma-server failed with {(e.__class__)} when creating user context for this user id."
            self.logger.error(error)
        return request
    def get_quicklink_flags(self):
        token, error = self.token_manager.get_token()
        headers = {
            "Authorization": f"Bearer {token}",
            "Content-Type": "application/json"
        }

        try:
            request = requests.get(self.url + "quicklinks", headers=headers)
        except requests.ConnectionError:
            error = "huma-server connection error"
            self.logger.error(error)
        except requests.Timeout:
            error = f"huma-server timed out when trying to post data."
            self.logger.error(error)
        except Exception as e:
            error = f"huma-server failed with {(e.__class__)} while getting quicklink flags."
            self.logger.error(error)
        return request

    def add_quicklink_flags(self,error_utterance_list):
        token, error = self.token_manager.get_token()
        headers = {
            "Authorization": f"Bearer {token}",
            "Content-Type": "application/json"
        }

        try:
            request = requests.post(self.url + "quicklinks", headers=headers,json={'error_utterance_list': error_utterance_list})
        except requests.ConnectionError:
            error = "huma-server connection error"
            self.logger.error(error)
        except requests.Timeout:
            error = f"huma-server timed out when trying to post data."
            self.logger.error(error)
        except Exception as e:
            error = f"huma-server failed with {(e.__class__)} while adding quicklink flags"
            self.logger.error(error)
        return request

    def delete_quicklink_flags(self,error_utterance_list):
        token, error = self.token_manager.get_token()
        headers = {
            "Authorization": f"Bearer {token}",
            "Content-Type": "application/json"
        }

        try:
            request = requests.get(self.url + "delete-quicklinks", headers=headers,json={'error_utterance_list': error_utterance_list})
        except requests.ConnectionError:
            error = "huma-server connection error"
            self.logger.error(error)
        except requests.Timeout:
            error = f"huma-server timed out when trying to post data."
            self.logger.error(error)
        except Exception as e:
            error = f"huma-server failed with {(e.__class__)} while deleting quicklink flags"
            self.logger.error(error)
        return request

    def delete_user_history(self,email):
        token, error = self.token_manager.get_token()
        headers = {
            "Authorization": f"Bearer {token}",
            "Content-Type": "application/json"
        }
        try:
            request = requests.get(self.url + "delete-history", headers=headers,json={'email': email})
        except requests.ConnectionError:
            error = "huma-server connection error"
            self.logger.error(error)
        except requests.Timeout:
            error = f"huma-server timed out when trying to post data."
            self.logger.error(error)
        except Exception as e:
            error = f"huma-server failed with {(e.__class__)} while deleting user history"
            self.logger.error(error)
        return request

    def clear_user_context(self,email):
        token, error = self.token_manager.get_token()
        headers = {
            "Authorization": f"Bearer {token}",
            "Content-Type": "application/json"
        }

        try:
            request = requests.get(self.url + "clear-context", headers=headers,json={'email': email})
        except requests.ConnectionError:
            error = "huma-server connection error"
            self.logger.error(error)
        except requests.Timeout:
            error = f"huma-server timed out when trying to post data."
            self.logger.error(error)
        except Exception as e:
            error = f"huma-server failed with {(e.__class__)} while clearing user context"
            self.logger.error(error)
        return request

    def get_maximum_vcpu(self):
        token, error = self.token_manager.get_token()
        headers = {
            "Authorization": f"Bearer {token}",
            "Content-Type": "application/json"
        }
        result = {}
        try:
            request = requests.get(self.url + "get-compute-maximum-vcpus", headers=headers)
            if request.status_code == 401:
                error = "auth error"
                self.logger.error(error)

            # convert to json format
            result = json.loads(request.text)
        except requests.ConnectionError:
            error = "huma-server connection error"
            self.logger.error(error)
        except requests.Timeout:
            error = f"huma-server timed out when trying to get data."
            self.logger.error(error)
        except Exception as e:
            error = f"huma-server failed with {(e.__class__)} when retrieving maximum vCPUs from compute environment"
            self.logger.error(error)
        # returning in JSON format
        return result

    def set_maximum_vcpu(self, maxvcpu: int):
        token, error = self.token_manager.get_token()
        headers = {
            "Authorization": f"Bearer {token}",
            "Content-Type": "application/json"
        }
        result = {}
        try:
            params = {
                "maximum_vcpus": maxvcpu
            }
            request = requests.post(self.url + "set-compute-maximum-vcpus", headers=headers, json=params)
            print(request)
            if request.status_code == 401:
                error = "auth error"
                self.logger.error(error)

            # convert to json format
            result = json.loads(request.text)
        except requests.ConnectionError:
            error = "huma-server connection error"
            self.logger.error(error)
        except requests.Timeout:
            error = f"huma-server timed out when trying to get data."
            self.logger.error(error)
        except Exception as e:
            error = f"huma-server failed with {(e.__class__)} when setting maximum vCPUs from compute environment"
            self.logger.error(error)
        # returning in JSON format
        return result


def get_utterance_test(utterance: str):
    huma_sdk_instantiated = huma_sdk(customer=CUSTOMER, environment=ENVIRONEMENT)
    utterances = []
    utterances.append(utterance)
    utterances.append(utterance + " new")
    u1_output = huma_sdk_instantiated.get_utterance(utterances[0])
    u2_output = huma_sdk_instantiated.get_utterance(utterances[1])
    return u1_output, u2_output


def ask_questions_test(questions: List[str]):
    huma_sdk_instantiated = huma_sdk(customer=CUSTOMER, environment=ENVIRONEMENT)
    outputs = {
        "job_ids": [],
        "answers": []
    }
    for question in questions:
        payload = huma_sdk_instantiated.submit_question(question)
        if not payload.get('answer_type') == 'cached':
            outputs['job_ids'].append(payload.get("job_id"))
        else:
            outputs['answers'].append({"status":  payload.get("status"), "visual": payload.get("visual")})
    return outputs


def get_answers_test(job_ids: List[str]):
    huma_sdk_instantiated = huma_sdk(customer=CUSTOMER, environment=ENVIRONEMENT)
    answers = []
    for job_id in job_ids:
        answer = huma_sdk_instantiated.get_answer(job_id)
        answers.append(answer)
    return answers


def test_up():
    # test of UP
    utterances = ["List insights related to drug Dupilumab",
                  "Show CRM Dashboard"]
    output_utterances = []
    for utterance in utterances:
        ua, ub = get_utterance_test(utterance)
        output_utterances.append({"utterance": ua,
                                  "utterance_new": ub})
    print(f"utterance test results: {output_utterances}")


def test_analyzer():
    # test of analyzer
    questions = ["List insights related to drug Dupilumab no answer cache",
                 "List insights related to drug Dupilumab",
                 "Show CRM Dashboard no answer cache"]
    output_answers = ask_questions_test(questions=questions)  # customer='014', env='dev'
    for answer in output_answers.get("answers"):
        print(answer)
    # wait for jobs to finish computing answers, put break point here and wait until jobs have succeeded
    time.sleep(5)
    # unanswered questions
    job_ids = output_answers.get('job_ids')
    answers = get_answers_test(job_ids=job_ids)
    for answer in answers:
        print(answer.text)


AUTO_SUGGEST_LIST = [
        "List competitor products mentioned for drug Cabazitaxel",
        "List insights related to Prostate Cancer",
        "List insights related to Multiple Myeloma",
        "Group strategic insights for drug Dupilumab",
        "Insights trending for Dupilumab in atopic dermatitis",
        "List competitor products mentioned for drug Cemiplimab",
        "Show CRM Dashboard",
        "Show Benchmark Dashboard",
        "Insights trending for Cemiplimab in NSCLC",
        "Rich and Henry were here!",
        "List competitor products mentioned for drug Cemiplimab for NSCLC",
        "List insights related to Nasal Polyposis",
        "Insights trending for Dupilumab in asthma",
        "Benchmark MSLs based on priority KOLs",
        "List competitor products mentioned for drug Alemtuzumab",
        "Insights trending for Dupilumab",
        "List insights for drug Alemtuzumab in another therapeutic area",
        "List insights related to drug Alemtuzumab",
        "List insights related to drug Cabazitaxel",
        "Benchmark MSLs by number of Unique HCPs",
        "List insights related to drug Sarilumab",
        "Benchmark MSLs by avg interaction duration",
        "Total number of interactions",
        "List insights related to Asthma",
        "Benchmark MSLs by # of insights generated",
        "List insights related to drug Dupilumab",
        "List insights related to Multiple Sclerosis",
        "List efficacy insights for drug Dupilumab",
        "List insights related to drug Cemiplimab",
        "List efficacy insights for drug Cemiplimab",
        "List insights for drug Dupilumab in another therapeutic area",
        "Insights trending for Cemiplimab",
        "List safety insights for Isatuximab",
        "Avg interaction duration",
        "List insights related to drug Isatuximab",
        "List competitor products mentioned for drug Dupilumab",
        "List insights related to Atopic Dermatitis"
    ]

def test_autosuggest():
    huma_sdk_instantiated = huma_sdk(customer=CUSTOMER, environment=ENVIRONEMENT)
    response = huma_sdk_instantiated.create_suggestions(AUTO_SUGGEST_LIST)
    assert response.status_code == 200
    partial_question = "List "
    suggestions = huma_sdk_instantiated.get_suggestions_query(partial_question)
    # access route in huma-server that will access route in autosuggestion service repo
    print(suggestions.get("suggestions"))


def test_audit_trail():
    huma_sdk_instantiated = huma_sdk(customer=CUSTOMER, environment=ENVIRONEMENT)
    audit = huma_sdk_instantiated.get_audit_trail()
    print(audit)
    # access route in huma-server that will access route in user-activity repo


ENVIRONEMENT="DEV"
CUSTOMER="014"

if __name__ == '__main__':
    #get_answers_test()
    test_up()
    #test_analyzer()
    # # point to your input json file with suggestions list
    #test_audit_trail()
    #test_autosuggest()
    pass
