from schema import Schema, And, Use, Optional, SchemaError
import os
from pathlib import Path
import os.path
from os import environ, path
import pickle
import pprint
import json
import requests
from botocore.exceptions import ClientError
from datetime import datetime, timedelta
import time
from huma_sdk.utils.log_utils import get_logger
import re
import getpass
from huma_sdk.config import Config
from copy import copy


class TokenStoreError(Exception):
    """Raised when unable to store a token"""
    pass


class TokenValidationError(Exception):
    """Raised when a token is not of a valid token format"""
    pass


class TokenManager:
    def __init__(self, customer=None, environment=None, url_override=Config.HUMA_API_URL, cli_token=Config.HUMA_CLI_KEY):
        self._log = get_logger(__name__)
        self._customer = customer
        self._environment = environment
        self._token_filename = self._setup_token_store()
        self._token_schema = self._get_token_schema()
        self._url = url_override
        self._cli_token = cli_token
        pass

    def _setup_token_store(self):
        if os.environ.get('AWS_LAMBDA_INITIALIZATION_TYPE', None):
            profile_directory = os.path.join("/tmp", ".huma")
        else:
            profile_directory = os.path.join(os.environ.get('USERPROFILE') or os.environ.get("HOME"), ".huma")
        Path(profile_directory).mkdir(parents=True, exist_ok=True)
        profile_file = os.path.join(profile_directory, f".tokens-{self._customer}-{self._environment}.pkl")
        if not path.isfile(profile_file):
            with open(profile_file, "wb") as f:
                empty_pickle = []
                pickle.dump(empty_pickle, f)
        return Path(profile_file)

    def validate_or_delete_token(self, tokens):
        try:
            return self._token_schema.validate(tokens)
        except SchemaError:
            return None

    def _store_tokens(self, tokens: list):
        try:
            with open(self._token_filename, 'wb') as f:
                    pickle.dump(tokens, f)
            return True
        except Exception as e:
            raise TokenStoreError(f"could not store updated or empty token because {e}")

    def _get_token_schema(self):
        return Schema([{
            'version': And(Use(int)),
            'username': And(Use(str)),
            'expiration': type(datetime(2018, 1, 22, 3, 20, 6)),
            'token': And(Use(str)),
            'customer': And(Use(str)),
            'environment': And(Use(str)),
            'refresh_token': And(Use(str)),
            'refresh_token_expiration': type(datetime(2018, 1, 22, 3, 20, 6)),
            'id_token': And(Use(str))
        }], ignore_extra_keys=True)

    def get_url(self):
        if self._url:
            return self._url
        
        url = f"https://huma-server.{self._environment}.{self._customer}.huma.ai/"
        return url

    def get_token(self):
        """
        Gets a token for a customer environment if it exists, or creates one and updates the token store

        Args:
            customer (str): the customer identifier like '009'
            environment (str): the environment identifier like 'dev', 'stage', or 'prod'

        Returns:
            str or None: a string of a token
        """
        if self._cli_token:
            return self._cli_token, None

        for token in self._get_tokens():
            if token.get("customer") == self._customer and \
                    token.get("environment") == self._environment:
                        if self.test_token_expiration(token.get("token")):
                            return token.get("token"), None
        new_token, error = self._get_new_auth()
        return new_token, error

    def _test_token_expiration(self, token: str):
        headers = {
            "Authorization": f"Bearer {token}",
            "Content-Type": "application/json"
        }
        token_response = requests.get(
            self.get_url() + "/authorize/validate_auth_token", headers=headers)
        return token_response

    def test_token_expiration(self, token: str):
        start_time = time.perf_counter()
        payload = None
        try:
            token_request = self._test_token_expiration(token=token)
        except requests.ConnectionError:
            error = "huma-server connection error"
            self._log.error(error)
        except requests.Timeout:
            error = f"huma-server timed out when trying to check token expiration."
            self._log.error(error)
        except Exception as e:
            error = f"huma-server failed with {(e.__class__)} when trying to check token_expiration becuase {e}"
            self._log.error(error)
        if token_request.status_code == 200:
            return True
        return False

    def _get_new_auth(self):

        headers = {
            "Content-Type": "application/json"
        }
        token = None
        error = None
        user = None
        password = None

        tokens = self._get_tokens()
        token_index = 0
        set_token_remove_indexes = []
        refreshable_token_info = {}
        for t in tokens:
            # if the current token is invalid and the refresh token is still valid
            if t.get("customer") == self._customer and \
                    t.get("environment") == self._environment and \
                    t.get("refresh_token_expiration") >= datetime.utcnow():
                        refreshable_token_info = {"index" : token_index, "token": t }
            if t.get("customer") == self._customer and \
                    t.get("environment") == self._environment and \
                    t.get("refresh_token_expiration") <= datetime.utcnow():
                        set_token_remove_indexes.append(token_index)
            token_index += 1

        for indx in set_token_remove_indexes:
            tokens.pop(indx)
        # our tokens in memory now contains only valid token.
        self._store_tokens(tokens)  # stored with refresh.
        # let's get a new token with our refresh token
        get_new_token = False
        if refreshable_token_info:
            payload = {
                "refresh_token": refreshable_token_info["token"].get("refresh_token")
            }
            try:
                auth_response = requests.post(self.get_url() +
                                              "authorize/get_auth", headers=headers, json=json.dumps(payload))
                # we'll save this payload at the end with the other tokens
                payload = json.loads(auth_response.text)
                if auth_response.status_code == 401:
                    get_new_token = True
                    if token_index:
                        tokens.pop(token_index - 1)
                        self._store_tokens(tokens)

            except requests.ConnectionError:
                error = "Huma server connection error"
            except requests.Timeout:
                error = f"Huma server timed out when trying to process a get token request"
            except Exception as e:
                error = f"Huma server failed with {(e.__class__)} when trying to process a get token request for a refresh token"

        if get_new_token or not refreshable_token_info:
            try:
                for login_attempt in range(0, 2):
                    user, password = self._get_user_pass()
                    payload = {
                        "username": user,
                        "password": password
                    }
                    auth_response = requests.post(self.get_url() +
                                                  "authorize/get_auth", headers=headers, json=json.dumps(payload))
                    # we'll save this payload at the end with the other tokens
                    payload = json.loads(auth_response.text)
                    # retry for auth error else provided valid token
                    if payload.get("auth"):
                        break
                    elif payload.get("msg"):
                        self._log.info(payload.get("msg"))
                        continue
                    else:
                        continue

            except requests.ConnectionError:
                error = "Huma server connection error"
            except requests.Timeout:
                error = f"Huma server timed out when trying to process a get token request"
            except Exception as e:
                error = f"Huma server failed with {(e.__class__)} when trying to process a new token request"

        if error:
            self._log.error(error)
            token = None
        else:
            if auth_response.status_code == 404:
                error = "The server returned a 404 error, Not found"
                self._log.error(error)
            elif auth_response.status_code == 500:
                error = "The server returned error 500, internal error."
                self._log.error(error)
            elif auth_response.status_code == 200:
                auth = payload["auth"]
                tokens = self._get_tokens()
                if refreshable_token_info:
                    token_index = refreshable_token_info.get("index")
                    tokens[token_index]["token"] = auth["AuthenticationResult"]["AccessToken"]
                    tokens[token_index]["token_expiration"] = datetime.utcnow() + timedelta(seconds=int(auth["AuthenticationResult"]["ExpiresIn"]) - 2)
                    tokens[token_index]["id_token"] = auth["AuthenticationResult"]["IdToken"]
                    token=tokens[token_index].get("token")
                else: # create a new token payload with username/pass generated token
                    token_payload = self._create_token_payload(auth=auth, user=user)
                    tokens.append(token_payload)
                    token = token_payload.get("token")
                self._store_tokens(tokens)
        return token, error

    def _get_tokens(self):
        valid_tokens = []
        try:
            if self._token_filename.exists() and os.path.getsize(self._token_filename) > 0:
                with open(self._token_filename, 'rb') as f:
                    unpickler = pickle.Unpickler(f)
                    tokens = unpickler.load()
                    if tokens:
                        valid_tokens.extend(tokens)
            elif self._token_filename.exists() and os.path.getsize(self._token_filename) == 0:
                os.remove(self._token_filename)
        except Exception as e:
            self._log.error(f"Failed to get tokens because {e}")
        return valid_tokens

    def _get_user_pass(self):
        user = input(f"Enter your username for {self._customer}/{self._environment}: ")
        password = getpass.getpass(f"Enter your password for {self._customer}/{self._environment}: ")
        return user, password

    def _create_token_payload(self, auth, user):
        return {
                'version': 1,
                'username': user,
                'expiration': datetime.utcnow() + timedelta(seconds=int(auth["AuthenticationResult"]["ExpiresIn"]) - 2),
                'token': auth["AuthenticationResult"]["AccessToken"],
                'customer': self._customer,
                'environment': self._environment,
                'refresh_token': auth["AuthenticationResult"]["RefreshToken"],
                'refresh_token_expiration': datetime.utcnow() + timedelta(days=30),
                'id_token': auth["AuthenticationResult"]["IdToken"]
            }

    def logout(self):
        tokens = self._get_tokens()
        token_copy = copy(tokens)
        token_index = 0
        for token in tokens:
            if token.get("customer") == self._customer and \
                    token.get("environment") == self._environment:
                user = token["username"]
                token_copy.pop(token_index)
                self._store_tokens(token_copy)
                self._log.info(f"User {user} has been logged out of {self._customer}-{self._environment}.")
                return True
            token_index += 1
        return False

    def login(self):
        token, error = self._get_new_auth()
        return token, error

def tests():
    sample_tokens = [{
        'version': 1,
        'username': 'rich@huma.ai',
        'expiration': datetime.utcnow() + timedelta(hours=1),
        'token': '2893ruksf',
        'customer': '009',
        'environment': 'dev',
        'refresh_token': 'oiawris',
        'refresh_token_expiration': datetime.utcnow() + timedelta(days=30),
        'ignored_key': 'this should not print',
        'id_token': 'asfifdsaoid'
    },
        {
        'version': 1,
        'username': 'rich@huma.ai',
        'expiration': datetime.utcnow() + timedelta(hours=1),
        'token': '28029384239898393ruksf',
        'customer': '008',
        'environment': 'dev',
        'refresh_token': '98laklfl;ksd',
        'refresh_token_expiration': datetime.utcnow() + timedelta(days=30),
        'id_token': 'asfifdsaoid'
    }]

    tm = TokenManager()
    # test 1 - validate tokens
    print(f"Test 1")
    validated_tokens = tm.validate_or_delete_token(sample_tokens)
    sample_tokens[0].pop('ignored_key')
    assert validated_tokens == sample_tokens
    # for token in sample_tokens:
    print(pprint.pformat(validated_tokens, indent=2))
    # test 2 - get token
    print(f"Test 2")
    customer, environment = '009', 'dev'
    token = tm.get_token(customer=customer, environment=environment)
    print(f"token for customer {customer} on environment {environment} is '{token}'")
    # test 3 - logout
    assert tm.logout(user="rich@huma.ai", customer='009', environment='dev')


if __name__ == '__main__':
    tests()
